<div class="controls">
    <?= /** @var \Icinga\Web\Widget\Tabs $tabs */ $tabs ?>
</div>

<div class="content">
    <?= /** @var \Icinga\Module\Graphite\Forms\Config\BackendForm $form */ $form ?>
</div>
